@@ -136,22 +136,6 @@ debugPrintf(const char *format, ...)
 }
 
 
-EXTERN_C void
-_assert(const char *_Message, const char *_File, unsigned _Line)
-{
-    debugPrintf("Assertion failed: %s, file %s, line %u\n", _Message, _File, _Line);
-    TerminateProcess(GetCurrentProcess(), 1);
-}
-
-
-EXTERN_C void
-_wassert(const wchar_t * _Message, const wchar_t *_File, unsigned _Line)
-{
-    debugPrintf("Assertion failed: %S, file %S, line %u\n", _Message, _File, _Line);
-    TerminateProcess(GetCurrentProcess(), 1);
-}
-
-
 static void
 MyCreateProcessCommon(BOOL bRet,
                       DWORD dwCreationFlags,
@@ -283,7 +267,7 @@ MyCreateProcessAsUserW(HANDLE hToken,
                                     lpProcessAttributes,
                                     lpThreadAttributes,
                                     bInheritHandles,
-                                    dwCreationFlags,
+                                    dwCreationFlags | CREATE_SUSPENDED,
                                     lpEnvironment,
                                     lpCurrentDirectory,
                                     lpStartupInfo,
