@@ -40,7 +40,7 @@
 #include "glstate.hpp"
 #include "glstate_internal.hpp"
 #include "halffloat.hpp"
-
+#include "floating_point_conversion.hpp"
 
 namespace glstate {
 
@@ -283,6 +283,26 @@ struct AttribDesc
     }
 };
 
+struct VertexAttribDesc : AttribDesc
+{
+    VertexAttribDesc()
+    {}
+
+    VertexAttribDesc(GLenum _type,
+                     GLint _size,
+                     GLint _stride)
+    {
+        type = _type;
+        size = 1;
+        elemType = _type;
+        elemStride = _gl_type_size(elemType);
+        numCols = _size;
+        numRows = 1;
+        rowStride = _stride ? _stride : numCols * elemStride;
+        colStride = elemStride;
+        arrayStride = rowStride;
+    }
+};
 
 static void
 dumpAttrib(StateWriter &writer,
@@ -354,6 +374,18 @@ dumpAttrib(StateWriter &writer,
             case GL_BOOL:
                 writer.writeBool(*u.uivalue);
                 break;
+            case GL_INT_2_10_10_10_REV:
+            case GL_UNSIGNED_INT_2_10_10_10_REV:
+                writer.writeInt(*u.ivalue & 0x3ff);
+                writer.writeInt((*u.ivalue >> 10) & 0x3ff);
+                writer.writeInt((*u.ivalue >> 20) & 0x3ff);
+                writer.writeInt((*u.ivalue >> 30) & 0x3);
+                break;
+            case GL_UNSIGNED_INT_10F_11F_11F_REV:
+                writer.writeFloat(uint11ToFLoat(*u.ivalue & 0x7ff));
+                writer.writeFloat(uint11ToFLoat((*u.ivalue >> 11) & 0x7ff));
+                writer.writeFloat(uint10ToFLoat((*u.ivalue >> 21) & 0x3ff));
+                break;
             default:
                 assert(0);
                 writer.writeNull();
@@ -874,7 +906,7 @@ dumpProgramUniformsStage(StateWriter &writer, GLint program, const char *stage)
 
 struct VertexAttrib {
     std::string name;
-    AttribDesc desc;
+    VertexAttribDesc desc;
     GLsizei offset = 0;
     GLsizei stride = 0;
     GLsizei size = 0;
@@ -951,7 +983,7 @@ dumpVertexAttributes(StateWriter &writer, Context &context, GLint program)
             size = 4;
         }
 
-        AttribDesc desc(type, size);
+        VertexAttribDesc desc(type, size, stride);
         if (!desc) {
             std::cerr << "warning: dumping of packed attribute (" << &name[0] << ") not yet supported\n";
             // TODO: handle
@@ -968,11 +1000,11 @@ dumpVertexAttributes(StateWriter &writer, Context &context, GLint program)
         }
 
         attrib.desc = desc;
-        GLsizei attribSize = attrib.desc.arrayStride;
+        GLsizei attribSize = attrib.desc.numCols * attrib.desc.colStride;
 
         if (stride == 0) {
             // tightly packed
-            stride = attribSize;
+            stride = attrib.desc.arrayStride;
         }
 
         attrib.offset = offset;
@@ -1010,7 +1042,7 @@ dumpVertexAttributes(StateWriter &writer, Context &context, GLint program)
     for (unsigned vertex = 0; vertex < count; ++vertex) {
         writer.beginObject();
         for (auto attrib : attribs) {
-            const AttribDesc & desc = attrib.desc;
+            const VertexAttribDesc & desc = attrib.desc;
             assert(desc);
 
             const GLbyte *vertex_data = attrib.map + attrib.stride*vertex + attrib.offset;
