@@ -400,12 +400,17 @@ class Retracer:
             self.checkOrigResult(function)
 
         self.deserializeArgs(function)
+
+        self.overrideArgs(function)
         
         self.declareRet(function)
         self.invokeFunction(function)
 
         self.swizzleValues(function)
 
+    def overrideArgs(self, function):
+        pass
+
     def retraceInterfaceMethodBody(self, interface, method):
         assert method.sideeffects
 
