@@ -474,89 +474,62 @@ D3DPV = Flags(DWORD, [
     "D3DPV_DONOTCOPYDATA",
 ])
 
-# XXX: Actually a mixture of enums and flags
+# D3DFVF is actually a mixture of enums and flags.  Flags must be sorted with
+# decreasing popcount, so names are properly matched.
 D3DFVF = Flags(DWORD, [
-    "D3DFVF_RESERVED0",
-    "D3DFVF_XYZ",
-    "D3DFVF_XYZRHW",
-    "D3DFVF_XYZB1",
-    "D3DFVF_XYZB2",
-    "D3DFVF_XYZB3",
-    "D3DFVF_XYZB4",
-    "D3DFVF_XYZB5",
-    "D3DFVF_NORMAL",
-    "D3DFVF_PSIZE",
-    "D3DFVF_DIFFUSE",
-    "D3DFVF_SPECULAR",
-    #"D3DFVF_TEX0",
-    #"D3DFVF_TEX1",
-    #"D3DFVF_TEX2",
-    #"D3DFVF_TEX3",
-    #"D3DFVF_TEX4",
-    #"D3DFVF_TEX5",
-    #"D3DFVF_TEX6",
-    #"D3DFVF_TEX7",
-    #"D3DFVF_TEX8",
-    "D3DFVF_LASTBETA_UBYTE4",
-    "D3DFVF_RESERVED2",
-    #"D3DFVF_TEXCOORDSIZE1(0)",
-    #"D3DFVF_TEXCOORDSIZE2(0)",
-    #"D3DFVF_TEXCOORDSIZE3(0)",
-    #"D3DFVF_TEXCOORDSIZE4(0)",
-    #"D3DFVF_TEXCOORDSIZE1(1)",
-    #"D3DFVF_TEXCOORDSIZE2(1)",
-    #"D3DFVF_TEXCOORDSIZE3(1)",
-    #"D3DFVF_TEXCOORDSIZE4(1)",
-    #"D3DFVF_TEXCOORDSIZE1(2)",
-    #"D3DFVF_TEXCOORDSIZE2(2)",
-    #"D3DFVF_TEXCOORDSIZE3(2)",
-    #"D3DFVF_TEXCOORDSIZE4(2)",
-    #"D3DFVF_TEXCOORDSIZE1(3)",
-    #"D3DFVF_TEXCOORDSIZE2(3)",
-    #"D3DFVF_TEXCOORDSIZE3(3)",
-    #"D3DFVF_TEXCOORDSIZE4(3)",
-])
+    "D3DFVF_RESERVED0",         # 0x00000001
 
-# XXX
-D3DVSD_TOKENTYPE = Enum("D3DVSD_TOKENTYPE", [
-    "D3DVSD_TOKEN_NOP",
-    "D3DVSD_TOKEN_STREAM",
-    "D3DVSD_TOKEN_STREAMDATA",
-    "D3DVSD_TOKEN_TESSELLATOR",
-    "D3DVSD_TOKEN_CONSTMEM",
-    "D3DVSD_TOKEN_EXT",
-    "D3DVSD_TOKEN_END",
-])
+    "D3DFVF_XYZB5",             # 0x0000000e
+    "D3DFVF_XYZB1",             # 0x00000006
+    "D3DFVF_XYZB3",             # 0x0000000a
+    "D3DFVF_XYZB4",             # 0x0000000c
+    "D3DFVF_XYZ",               # 0x00000002
+    "D3DFVF_XYZRHW",            # 0x00000004
+    "D3DFVF_XYZB2",             # 0x00000008
 
-# XXX
-D3DVSDT = FakeEnum(DWORD, [
-    "D3DVSDT_FLOAT1",
-    "D3DVSDT_FLOAT2",
-    "D3DVSDT_FLOAT3",
-    "D3DVSDT_FLOAT4",
-    "D3DVSDT_D3DCOLOR",
-    "D3DVSDT_UBYTE4",
-    "D3DVSDT_SHORT2",
-    "D3DVSDT_SHORT4",
-    "D3DVSDE_POSITION",
-    "D3DVSDE_BLENDWEIGHT",
-    "D3DVSDE_BLENDINDICES",
-    "D3DVSDE_NORMAL",
-    "D3DVSDE_PSIZE",
-    "D3DVSDE_DIFFUSE",
-    "D3DVSDE_SPECULAR",
-    "D3DVSDE_TEXCOORD0",
-    "D3DVSDE_TEXCOORD1",
-    "D3DVSDE_TEXCOORD2",
-    "D3DVSDE_TEXCOORD3",
-    "D3DVSDE_TEXCOORD4",
-    "D3DVSDE_TEXCOORD5",
-    "D3DVSDE_TEXCOORD6",
-    "D3DVSDE_TEXCOORD7",
-    "D3DVSDE_POSITION2",
-    "D3DVSDE_NORMAL2",
-    "D3DDP_MAXTEXCOORD",
-    "D3DSI_OPCODE_MASK",
+    "D3DFVF_NORMAL",            # 0x00000010
+    "D3DFVF_PSIZE",             # 0x00000020
+    "D3DFVF_DIFFUSE",           # 0x00000040
+    "D3DFVF_SPECULAR",          # 0x00000080
+
+    "D3DFVF_TEX7",              # 0x00000700
+    "D3DFVF_TEX3",              # 0x00000300
+    "D3DFVF_TEX5",              # 0x00000500
+    "D3DFVF_TEX6",              # 0x00000600
+    "D3DFVF_TEX1",              # 0x00000100
+    "D3DFVF_TEX2",              # 0x00000200
+    "D3DFVF_TEX4",              # 0x00000400
+    "D3DFVF_TEX8",              # 0x00000800
+#   "D3DFVF_TEX0",              # 0x00000000 (implied by omission)
+
+    "D3DFVF_LASTBETA_UBYTE4",   # 0x00001000
+    "D3DFVF_RESERVED2",         # 0x0000e000
+
+    "D3DFVF_TEXCOORDSIZE1(0)",  # 0x00030000
+    "D3DFVF_TEXCOORDSIZE3(0)",  # 0x00010000
+    "D3DFVF_TEXCOORDSIZE4(0)",  # 0x00020000
+    "D3DFVF_TEXCOORDSIZE1(1)",  # 0x000c0000
+    "D3DFVF_TEXCOORDSIZE3(1)",  # 0x00040000
+    "D3DFVF_TEXCOORDSIZE4(1)",  # 0x00080000
+    "D3DFVF_TEXCOORDSIZE1(2)",  # 0x00300000
+    "D3DFVF_TEXCOORDSIZE3(2)",  # 0x00100000
+    "D3DFVF_TEXCOORDSIZE4(2)",  # 0x00200000
+    "D3DFVF_TEXCOORDSIZE1(3)",  # 0x00c00000
+    "D3DFVF_TEXCOORDSIZE3(3)",  # 0x00400000
+    "D3DFVF_TEXCOORDSIZE4(3)",  # 0x00800000
+    "D3DFVF_TEXCOORDSIZE1(4)",  # 0x03000000
+    "D3DFVF_TEXCOORDSIZE3(4)",  # 0x01000000
+    "D3DFVF_TEXCOORDSIZE4(4)",  # 0x02000000
+    "D3DFVF_TEXCOORDSIZE1(5)",  # 0x0c000000
+    "D3DFVF_TEXCOORDSIZE3(5)",  # 0x04000000
+    "D3DFVF_TEXCOORDSIZE4(5)",  # 0x08000000
+    "D3DFVF_TEXCOORDSIZE1(6)",  # 0x30000000
+    "D3DFVF_TEXCOORDSIZE3(6)",  # 0x10000000
+    "D3DFVF_TEXCOORDSIZE4(6)",  # 0x20000000
+#   "D3DFVF_TEXCOORDSIZE1(7)",  # 0xc0000000 (narrowing conversion)
+#   "D3DFVF_TEXCOORDSIZE3(7)",  # 0x40000000
+#   "D3DFVF_TEXCOORDSIZE4(7)",  # 0x80000000 (narrowing conversion)
+#   "D3DFVF_TEXCOORDSIZE2(x)",  # 0x00000000 (implied by omission)
 ])
 
 D3DBASISTYPE = Enum("D3DBASISTYPE", [
