@@ -167,6 +167,7 @@ GLXEnum = FakeEnum(Int, [
     "GLX_GPU_NUM_SIMD_AMD",		# 0x21A6
     "GLX_GPU_NUM_RB_AMD",		# 0x21A7
     "GLX_GPU_NUM_SPI_AMD",		# 0x21A8
+    "GLX_CONTEXT_OPENGL_NO_ERROR_ARB",		# 0x31B3
     "GLX_NONE",		# 0x8000
     "GLX_SLOW_CONFIG",		# 0x8001
     "GLX_TRUE_COLOR",		# 0x8002
@@ -298,8 +299,8 @@ GLXCommonAttribs = [
     ('GLX_TRANSPARENT_GREEN_VALUE', Int),
     ('GLX_TRANSPARENT_BLUE_VALUE', Int),
     ('GLX_TRANSPARENT_ALPHA_VALUE', Int),
-    ('GLX_SAMPLE_BUFFERS', UInt),
-    ('GLX_SAMPLES', UInt),
+    ('GLX_SAMPLE_BUFFERS', Int),
+    ('GLX_SAMPLES', Int),
 ]
 
 GLXVisualAttribs = AttribArray(GLXEnum, GLXCommonAttribs + [
@@ -350,6 +351,7 @@ GLXContextARBAttribs = AttribArray(Const(GLXEnum), [
         'GLX_CONTEXT_RELEASE_BEHAVIOR_NONE_ARB',
         'GLX_CONTEXT_RELEASE_BEHAVIOR_FLUSH_ARB',
     ])),
+    ('GLX_CONTEXT_OPENGL_NO_ERROR_ARB', Bool),
 ])
 
 GLXPixmapAttribs = AttribArray(Const(GLXEnum), [
