@@ -149,6 +149,7 @@ class GlTracer(Tracer):
         print('#include "gltrace.hpp"')
         print('#include "gltrace_arrays.hpp"')
         print('#include "glmemshadow.hpp"')
+        print('#include "gltrace_unpack_compressed.hpp"')
         print()
 
         # Whether we need user arrays
@@ -209,7 +210,7 @@ class GlTracer(Tracer):
             if function.name in self.fake_function_names:
                 print(function.prototype('_fake_' + function.name) + ';')
         print()
-        print(r'static inline void')
+        print(r'inline void')
         print(r'_fakeStringMarker(const std::string &s) {')
         print(r'    _fake_glStringMarkerGREMEDY(s.length(), s.data());')
         print(r'}')
@@ -408,7 +409,7 @@ class GlTracer(Tracer):
             print('        static bool warned = false;')
             print('        if (!warned) {')
             print('            warned = true;')
-            print('            os::log("apitrace: warning: %s: call will be faked due to pointer to user memory (https://github.com/apitrace/apitrace/blob/master/docs/BUGS.markdown#tracing)\\n", __FUNCTION__);')
+            print('            os::log("apitrace: warning: %s: call will be faked due to pointer to user memory (https://git.io/JOMRv)\\n", __FUNCTION__);')
             print('        }')
             print('        gltrace::Context *_ctx = gltrace::getContext();')
             print('        _ctx->user_arrays = true;')
@@ -466,7 +467,7 @@ class GlTracer(Tracer):
                 print(r'            GLint _size = 0;')
                 print(r'            _%s(%s%s, &_size);' % (getter, extraArg, pname))
                 print(r'            if (_size != GL_BGRA) {')
-                print(r'                os::log("apitrace: warning: %s(%s) does not return GL_BGRA; trace will be incorrect (https://github.com/apitrace/apitrace/issues/261)\n");' % (getter, pname))
+                print(r'                os::log("apitrace: warning: %s(%s) does not return GL_BGRA; trace will be incorrect (https://git.io/JOM0n)\n");' % (getter, pname))
                 print(r'            }')
                 print(r'            _checked = true;')
                 print(r'        }')
@@ -525,7 +526,7 @@ class GlTracer(Tracer):
         if function.name.startswith('glArrayElement'):
             print(r'    gltrace::Context *_ctx = gltrace::getContext();')
             print(r'    if (_ctx->userArraysOnBegin) {')
-            print(r'        os::log("apitrace: warning: user arrays with glArrayElement not supported (https://github.com/apitrace/apitrace/issues/276)\n");')
+            print(r'        os::log("apitrace: warning: user arrays with glArrayElement not supported (https://git.io/JOM0l)\n");')
             print(r'        _ctx->userArraysOnBegin = false;')
             print(r'    }')
         
@@ -553,8 +554,8 @@ class GlTracer(Tracer):
             print('    if ((access_flags & GL_MAP_COHERENT_BIT) && (access_flags & GL_MAP_WRITE_BIT)) {')
             print('        gltrace::Context *_ctx = gltrace::getContext();')
             print('        GLint buffer = getBufferName(target);')
-            print('        auto it = _ctx->bufferToShadowMemory.find(buffer);')
-            print('        if (it != _ctx->bufferToShadowMemory.end()) {')
+            print('        auto it = _ctx->sharedRes->bufferToShadowMemory.find(buffer);')
+            print('        if (it != _ctx->sharedRes->bufferToShadowMemory.end()) {')
             print('            it->second->unmap(trace::fakeMemcpy);')
             print('        } else {')
             print(r'            os::log("apitrace: error: %s: cannot find memory shadow\n", __FUNCTION__);')
@@ -625,8 +626,8 @@ class GlTracer(Tracer):
             print('    _glGetNamedBufferParameteriv(buffer, GL_BUFFER_ACCESS_FLAGS, &access_flags);')
             print('    if ((access_flags & GL_MAP_COHERENT_BIT) && (access_flags & GL_MAP_WRITE_BIT)) {')
             print('        gltrace::Context *_ctx = gltrace::getContext();')
-            print('        auto it = _ctx->bufferToShadowMemory.find(buffer);')
-            print('        if (it != _ctx->bufferToShadowMemory.end()) {')
+            print('        auto it = _ctx->sharedRes->bufferToShadowMemory.find(buffer);')
+            print('        if (it != _ctx->sharedRes->bufferToShadowMemory.end()) {')
             print('            it->second->unmap(trace::fakeMemcpy);')
             print('        } else {')
             print(r'            os::log("apitrace: error: %s: cannot find memory shadow\n", __FUNCTION__);')
@@ -646,8 +647,8 @@ class GlTracer(Tracer):
             print('    _glGetNamedBufferParameterivEXT(buffer, GL_BUFFER_ACCESS_FLAGS, &access_flags);')
             print('    if ((access_flags & GL_MAP_COHERENT_BIT) && (access_flags & GL_MAP_WRITE_BIT)) {')
             print('        gltrace::Context *_ctx = gltrace::getContext();')
-            print('        auto it = _ctx->bufferToShadowMemory.find(buffer);')
-            print('        if (it != _ctx->bufferToShadowMemory.end()) {')
+            print('        auto it = _ctx->sharedRes->bufferToShadowMemory.find(buffer);')
+            print('        if (it != _ctx->sharedRes->bufferToShadowMemory.end()) {')
             print('            it->second->unmap(trace::fakeMemcpy);')
             print('        } else {')
             print(r'            os::log("apitrace: error: %s: cannot find memory shadow\n", __FUNCTION__);')
@@ -712,7 +713,7 @@ class GlTracer(Tracer):
             print(r'        auto memoryShadow = std::make_unique<GLMemoryShadow>();')
             print(r'        const bool success = memoryShadow->init(data, size);')
             print(r'        if (success) {')
-            print(r'            _ctx->bufferToShadowMemory.insert(std::make_pair(buffer, std::move(memoryShadow)));')
+            print(r'            _ctx->sharedRes->bufferToShadowMemory.insert(std::make_pair(buffer, std::move(memoryShadow)));')
             print(r'        } else {')
             print(r'            os::log("apitrace: error: %s: cannot create memory shadow\n", __FUNCTION__);')
             print(r'        }')
@@ -910,8 +911,8 @@ class GlTracer(Tracer):
             print(r'        gltrace::Context *_ctx = gltrace::getContext();')
             if function.name in ('glMapBufferRange', 'glMapBufferRangeEXT'):
                 print(r'        GLint buffer = getBufferName(target);')
-            print(r'        auto it = _ctx->bufferToShadowMemory.find(buffer);')
-            print(r'        if (it != _ctx->bufferToShadowMemory.end()) {')
+            print(r'        auto it = _ctx->sharedRes->bufferToShadowMemory.find(buffer);')
+            print(r'        if (it != _ctx->sharedRes->bufferToShadowMemory.end()) {')
             print(r'            _result = it->second->map(_ctx, _result, access, offset, length);')
             print(r'        } else {')
             print(r'            os::log("apitrace: error: %s: cannot find memory shadow\n", __FUNCTION__);')
@@ -923,17 +924,20 @@ class GlTracer(Tracer):
         # GPU are done and buffers are updated.
         if function.name == 'glWaitSync':
             print(r'    gltrace::Context *_ctx = gltrace::getContext();')
+            print(r'    GLMemoryShadow::commitAllWrites(_ctx, trace::fakeMemcpy);')
             print(r'    GLMemoryShadow::syncAllForReads(_ctx);')
 
         if function.name == 'glClientWaitSync':
             print(r'    if (_result == GL_ALREADY_SIGNALED || _result == GL_CONDITION_SATISFIED) {')
             print(r'        gltrace::Context *_ctx = gltrace::getContext();')
+            print(r'        GLMemoryShadow::commitAllWrites(_ctx, trace::fakeMemcpy);')
             print(r'        GLMemoryShadow::syncAllForReads(_ctx);')
             print(r'    }')
 
         if function.name == 'glGetSynciv':
             print(r'    if (pname == GL_SYNC_STATUS && bufSize > 0 && values[0] == GL_SIGNALED) {')
             print(r'        gltrace::Context *_ctx = gltrace::getContext();')
+            print(r'        GLMemoryShadow::commitAllWrites(_ctx, trace::fakeMemcpy);')
             print(r'        GLMemoryShadow::syncAllForReads(_ctx);')
             print(r'    }')
 
@@ -975,6 +979,8 @@ class GlTracer(Tracer):
         r'(Compressed)?(Multi)?Tex(ture)?(Sub)?Image[1-4]D',
     ]) + r')[0-9A-Z]*$')
 
+    compressed_image_function_regex = re.compile(r'^glCompressedTex(ture)?(Sub)?Image[1-4]D[0-9A-Z]*$')
+
     def serializeArgValue(self, function, arg):
         # Recognize offsets instead of blobs when a PBO is bound
         if self.unpack_function_regex.match(function.name) \
@@ -989,7 +995,10 @@ class GlTracer(Tracer):
             print('        if (_unpack_buffer) {')
             print('            trace::localWriter.writePointer((uintptr_t)%s);' % arg.name)
             print('        } else {')
-            Tracer.serializeArgValue(self, function, arg)
+            if self.compressed_image_function_regex.match(function.name):
+                print('            %s;' % arg.type.size.format('[](const void* data, GLsizei size){ trace::localWriter.writeBlob(data, size); }'))
+            else:
+                Tracer.serializeArgValue(self, function, arg)
             print('        }')
             print('    }')
             return
